Package: PGRdup
Title: Discover Probable Duplicates in Plant Genetic Resources
        Collections
Version: 0.3.0
Authors@R: c(
    person(given = "J.", family = "Aravind",
           email = "j.aravind@icar.org.in",
           role = c("aut", "cre"),
		       comment = c(ORCID = "0000-0002-4791-442X")),
    person(given = "J.", family = "Radhamani",
           email = "jalli.radhamani@icar.gov.in",
		       role = "aut"),
    person(family = c("Kalyani Srinivasan"),
           email = "kalyani.srinivasan@icar.gov.in",
		       role = "aut"),
    person(given = "B.", family = c("Ananda Subhash"),
           email = "anandasubhash@gmail.com",
		       role = "aut"),
    person(given = c("Rishi", "Kumar"), family = "Tyagi",
           email = "rishi.tyagi@icar.gov.in",
		       role = "aut"),
    person("ICAR-NBGPR",
           role = "cph",
		       comment = c(ROR = "https://ror.org/00scbd467",
		                   url = "https://nbpgr.org.in/")),
    person(given = "Maurice", family = "Aubrey",
           email = "maurice@hevanet.com",
           comment = "Double Metaphone",
           role = "ctb"),
    person(given = "Kevin", family = "Atkinson",
           email = "kevina@users.sourceforge.net",
           comment = "Double Metaphone",
           role = "ctb"),
    person(given = "Lawrence", family = "Philips",
           comment = "Double Metaphone",
           role = "ctb"))
Description: Provides functions to aid the identification of probable/possible
    duplicates in Plant Genetic Resources (PGR) collections using
    'passport databases' comprising of information records of each constituent
    sample. These include methods for cleaning the data, creation of a
    searchable Key Word in Context (KWIC) index of keywords associated with
    sample records and the identification of nearly identical records with
    similar information by fuzzy, phonetic and semantic matching of keywords.
Depends: R (>= 4.1.0)
Imports: data.table (>= 1.9.3), igraph, stringdist (>= 0.9.4), stringi,
        ggplot2, grid, gridExtra, methods, utils, stats
Suggests: diagram, wordcloud, microbenchmark, XML, httr, RCurl, knitr,
        rmarkdown, pander
Copyright: 2014-2026, ICAR-NBPGR
License: GPL-2 | GPL-3
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr
RoxygenNote: 7.3.3
URL: https://cran.r-project.org/package=PGRdup,
        https://github.com/aravind-j/PGRdup,
        https://doi.org/10.5281/zenodo.841963,
        https://aravind-j.github.io/PGRdup/,
        https://www.rdocumentation.org/packages/PGRdup
BugReports: https://github.com/aravind-j/PGRdup/issues
NeedsCompilation: yes
Packaged: 2026-04-21 10:51:52 UTC; Aravind-DGC
Author: J. Aravind [aut, cre] (ORCID: <https://orcid.org/0000-0002-4791-442X>),
  J. Radhamani [aut],
  Kalyani Srinivasan [aut],
  B. Ananda Subhash [aut],
  Rishi Kumar Tyagi [aut],
  ICAR-NBGPR [cph] (ROR: <https://ror.org/00scbd467>, url:
    https://nbpgr.org.in/),
  Maurice Aubrey [ctb] (Double Metaphone),
  Kevin Atkinson [ctb] (Double Metaphone),
  Lawrence Philips [ctb] (Double Metaphone)
Maintainer: J. Aravind <j.aravind@icar.org.in>
Repository: CRAN
Date/Publication: 2026-04-21 22:22:29 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-28 08:18:04 UTC; unix
Archs: PGRdup.so.dSYM
