# Generated by roxygen2: do not edit by hand

S3method(print,KWIC)
S3method(print,ProbDup)
export(AddProbDup)
export(DataClean)
export(DisProbDup)
export(DoubleMetaphone)
export(KWCounts)
export(KWIC)
export(MergeKW)
export(MergePrefix)
export(MergeProbDup)
export(MergeSuffix)
export(ParseProbDup)
export(ProbDup)
export(ReconstructProbDup)
export(ReviewProbDup)
export(SplitProbDup)
export(ValidatePrimKey)
export(ViewProbDup)
export(print.KWIC)
export(print.ProbDup)
export(read.genesys)
import(data.table)
import(igraph)
import(stringdist)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,coord_polar)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplotGrob)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_y_reverse)
importFrom(ggplot2,theme)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(gridExtra,arrangeGrob)
importFrom(methods,is)
importFrom(stats,embed)
importFrom(stringi,stri_count)
importFrom(stringi,stri_count_fixed)
importFrom(stringi,stri_count_regex)
importFrom(stringi,stri_split_fixed)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,setTxtProgressBar)
importFrom(utils,stack)
importFrom(utils,txtProgressBar)
importFrom(utils,unzip)
useDynLib(PGRdup, .registration = TRUE)
