### This file is part of 'PGRdup' package for R.

### Copyright (C) 2014-2026, ICAR-NBPGR.
#
# PGRdup is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# PGRdup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/



#' Functions to facilitate genebank managers in the identification of probable 
#' duplicate accessions from plant genetic resources (PGR) passport databases.
#' 
#' @title The PGRdup Package
#' @name PGRdup-package
#' @aliases PGRdup
#' @author J Aravind \email{j.aravind@@icar.org.in} \cr J Radhamani 
#'   \email{jalli.radhamani@@icar.gov.in} \cr Kalyani Srinivasan 
#'   \email{kalyani.srinivasan@@icar.gov.in} \cr B Ananda Subhash 
#'   \email{anandasubhash@@gmail.com} \cr RK Tyagi
#'   \email{rishi.tyagi@@icar.gov.in}
#' @import data.table
#' @import igraph
#' @import stringdist
#' @importFrom stringi stri_split_fixed
#' @keywords internal
"_PACKAGE"
