### This file is part of 'PGRdup' package for R.

### Copyright (C) 2014-2026, ICAR-NBPGR.
#
# PGRdup is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# PGRdup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/



if (getRversion() >= "2.15.1")  {
  utils::globalVariables(
    c(
      ".", "COMBINED", "COUNT", "DEL", "exep", "FACTOR",
      "FIELD", "fixed", "Freq", "FuzzydupID",
      "FuzzydupIDKW", "FuzzydupIDKW2", "genesysId", "ID",
      "IDKW", "ind", "iter", "K", "K1_PRIM_ID",
      "K2_PRIM_ID", "KEYWORD", "KIND", "KIND2", "KW1",
      "kwic", "label", "name", "Ndup", "NO.ACC", "NO.SETS",
      "PhoneticdupID", "PhoneticdupIDKW", "PRIM_ID",
      "primdup", "SemanticdupID", "SemanticdupIDKW", "Seq",
      "SET_NO", "shape", "shape2", "singles", "SPLIT",
      "TEMP", "Total", "TYPE", "V1", "value",
      "variable", "WITHIN", "ymax", "ymin",
      "digit", "enc", "idkw", "keyword", "synset"
    )
  )
}
